import { Component } from '@angular/core';
import { events } from '@minsky/shared';
import { ElectronService } from '@minsky/core';
import { ActivatedRoute } from '@angular/router';
import { switchMap } from 'rxjs/operators';
import { ScaleHandler } from '../scale-handler/scale-handler.class';
import { uniqBy, sortBy } from 'lodash-es';

@Component({
  selector: 'minsky-parameters',
  templateUrl: './parameters.component.html',
  styleUrls: ['./parameters.component.scss'],
})
export class ParametersComponent {
  variables;

  tensorText = '<tensor>';

  type: string;

  scale = new ScaleHandler();

  constructor(private electronService: ElectronService, route: ActivatedRoute) {
    route.params.pipe(switchMap(p => {
      this.type = p['tab'];
      if(this.type === 'parameters') {
        return electronService.minsky.parameterTab.getDisplayVariables();
      } else {
        return electronService.minsky.variableTab.getDisplayVariables();
      }
    })).subscribe((v: any) => this.prepareVariables(v));
  }

  prepareVariables(variables: any[]) {
    // why tf is this uniqBy necessary?! sort duplicates rows..? js' own .sort function does the same thing
    this.variables = uniqBy(sortBy(variables, ['type', 'name']), v => v.name);
  }

  changeScale(e) {
    if(e.ctrlKey) {
      this.scale.changeScale(e.deltaY);
    }
  }

  truncateValue(value: string) {
    if(isNaN(+value)) return value;
    const stringDecimals = String(+value - Math.floor(+value));
    if(stringDecimals.length > 6) return (+value).toFixed(4);
    return String(value);
  }

  contextMenu(valueId: string) {
    this.electronService.log(`sending context menu: ${valueId}`);
    this.electronService.send(events.CONTEXT_MENU, {
      type: 'html-summary',
      command: valueId,
    });
  }
}
